#!/usr/bin/env python3

from collections import OrderedDict
import re
import sys

from bs4 import BeautifulSoup
import requests


def match(row):
    if len(row) >= 6 \
            and re.match(r'[A-Z0-9_]', row[0]) \
            and re.match(r'[A-F0-9]{4}', row[2]):
        return True
    return False


def labels(html_doc=None):
    if not html_doc:
        r = requests.get('https://newgrf-specs.tt-wiki.net/wiki/CargoTypes')
        html_doc = r.text
    soup = BeautifulSoup(html_doc, 'html.parser')
    table = soup.find('table')
    cargos = OrderedDict()
    default_cargo = True
    for tr in table.find_all('tr'):
        row = list(td.get_text().strip() for td in tr.find_all('td'))
        if row and re.match(r'special cargos', row[0], re.IGNORECASE):
            break
        if row and re.match(r'new cargos', row[0], re.IGNORECASE):
            default_cargo = False
        if not match(row):
            continue
        label = row[0]
        desc = row[1]
        cc = int(row[2][:4], base=16)
        # Some cargos have different classes in ecs/firs/yeti
        # Some of them are in the notes of the wiki page,
        # try to get the firs classes from these notes
        if len(row) >= 7:
            m = re.search(r'firs[-:., ]+([A-F0-9]{4})', row[6], re.IGNORECASE)
            if m:
                cc = int(m.group(1), base=16)
        industries = []
        if default_cargo:
            industries.append('Default')
        for td in row[3:6]:
            if td:
                industries.append(td)

        cargos[label] = {'description': desc, 'cargo_class': cc,
                         'industries': industries}

    return cargos


def main():
    if len(sys.argv) > 1:
        with open(sys.argv[1], encoding='utf-8') as f:
            html_doc = f.read()
    else:
        html_doc=None
    lbs = labels(html_doc)
    for k in lbs:
        print(k, lbs[k])


if __name__ == '__main__':
    main()
